unit MainUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.UIConsts,System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects3D, FMX.Types3D,
  FMX.Layouts, FMX.Memo, IBX.IBDatabase, Data.DB, IBX.IBCustomDataSet, IBX.IBQuery,
  FMX.Layers3D, Data.Bind.Components, Data.Bind.DBScope, FMX.Materials,
  FMX.Objects, FMX.MaterialSources, FMX.StdCtrls, FMX.Controls3D, FMX.Viewport3D,
  System.Math.Vectors, FMX.Controls.Presentation;

const
  BrazilStateMax = 27; // # of Brazil States + Brasilia
  PopulationDataYearsMax = 13;  // # of years where we have population data
  ScalingConstant = 1.3;  // used to scale the Path3D(s)
  PopulationScaleFactor = 100000.0; // used to set the population depth


type
  TForm2 = class(TForm)
    Viewport3D1: TViewport3D;
    OpenStatesButton: TButton;
    Camera1: TCamera;
    BrazilDatabase: TIBDatabase;
    IBTransaction1: TIBTransaction;
    StatesQuery: TIBQuery;
    StatesQuerySTATE_ABBREVIATION: TIBStringField;
    StatesQuerySTATE_NAME: TIBStringField;
    StatesQueryPATH_DATA: TWideMemoField;
    StatesQuerySTATE_GRAPHIC: TBlobField;
    Path3D1: TPath3D;
    YearsTrackBar: TTrackBar;
    PopYearLabel: TLabel;
    Timer1: TTimer;
    CloseStatesButton: TButton;
    ScaledLayout1: TScaledLayout;
    Panel1: TPanel;
    Layout3D1: TLayout3D;
    ResetCameraButton: TButton;
    Light1: TLight;
    PopQuery: TIBQuery;
    PopDataSource: TDataSource;
    BindScopeDB1: TBindScopeDB;
    PopQuerySTATE_ABBREVIATION: TIBStringField;
    PopQueryCENSUS_YEAR: TIBStringField;
    PopQueryPOPULATION: TIBStringField;
    ColorMaterialSource1: TColorMaterialSource;
    Layer3D1: TLayer3D;
    Panel3: TPanel;
    sbDown: TSpeedButton;
    Path5: TPath;
    sbLeft: TSpeedButton;
    Path6: TPath;
    spRight: TSpeedButton;
    Path7: TPath;
    spUp: TSpeedButton;
    Path8: TPath;
    tbZoom: TTrackBar;
    ArcDial2: TArcDial;
    procedure OpenStatesButtonClick(Sender: TObject);
    procedure CloseStatesButtonClick(Sender: TObject);
    procedure YearsTrackBarChange(Sender: TObject);
    procedure ResetCameraButtonClick(Sender: TObject);
    procedure ArcDial2Change(Sender: TObject);
    procedure spUpMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure spRightMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure sbLeftMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure sbDownMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure tbZoomChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
  private
    { Private declarations }
    PopulationIndex : integer;
    Btn : integer;
    CamVector : TVector3D;
    StateArray : array [0..BrazilStateMax-1] of record
      StateAbbreviation : string;
      StateName : string;
      StatePath3D : TPath3D;
      StateMaterialSource : TColorMaterialSource;
      // StateGraphic : TJpeg;
    end;
    procedure SetStatePopulationDepth();
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

  PopulationYears :  array[0..PopulationDataYearsMax-1] of string
    = ('1872','1890','1900','1920',
       '1940','1950','1960','1970',
       '1980','1991','2000','2010',
       '2020');

  StatesColors : array[0..BrazilStateMax-1] of TAlphaColor
    = (claBlue,claBrown,claDarkgreen,
       claCrimson,claCoral,claDarkorange,
       claSlateBlue,claDarkviolet,claForestgreen,
       claFuchsia,claGreen,claBlue,
       claIndigo,claMagenta,claMaroon,
       claNavy,claOrange,claLime,
       claPurple,claRed,claSalmon,
       claSpringgreen,claSteelBlue,claYellow,
       claTomato,claSienna,claTomato
      );

implementation

{$R *.fmx}

procedure TForm2.OpenStatesButtonClick(Sender: TObject);
var
  i : integer;
  BR : TRectF;
begin
  OpenStatesButton.Enabled := false;
  CloseStatesButton.Enabled := true;
  YearsTrackBar.Enabled := true;
  PopYearLabel.Enabled := true;
  i := 0;
  Randomize;
  BrazilDataBase.Open;
  StatesQuery.Open;
  while not StatesQuery.Eof do begin
    StateArray[i].StateAbbreviation := StatesQuerySTATE_ABBREVIATION.AsString;
    StateArray[i].StateName := StatesQuerySTATE_NAME.AsString;
    StateArray[i].StatePath3D := TPath3D.Create(self);
    StateArray[i].StatePath3D.Projection := TProjection.Screen;

    StateArray[i].StatePath3D.Parent := Layout3D1;
    // TODO - have state bitmaps in the db ->StateArray[i].StateGraphic := StatesQuerySTATE_GRAPHIC.AsBytes;
    StateArray[i].StatePath3D.Path.Data := StatesQueryPATH_DATA.AsString;

    // get bounded rectangle for the state
    BR := StateArray[i].StatePath3D.Path.GetBounds;
    StateArray[i].StatePath3D.Position.X := (BR.Left+BR.Width/2)/ScalingConstant;
    StateArray[i].StatePath3D.Position.Y := (BR.Top+BR.Height/2)/ScalingConstant;
    StateArray[i].StatePath3D.Height := BR.Height/ScalingConstant;
    StateArray[i].StatePath3D.Width := BR.Width/ScalingConstant;
    StateArray[i].StatePath3D.Depth := 0;

    // Set material, back and shaft colors for the state
    StateArray[i].StateMaterialSource := TColorMaterialSource.Create(self);
    StateArray[i].StateMaterialSource.Parent := StateArray[i].StatePath3D;
    StateArray[i].StateMaterialSource.Color := StatesColors[i];
    StateArray[i].StatePath3D.MaterialSource := StateArray[i].StateMaterialSource;
    StateArray[i].StatePath3D.MaterialShaftSource := StateArray[i].StateMaterialSource;
    StateArray[i].StatePath3D.MaterialBackSource := StateArray[i].StateMaterialSource;

    Inc(i);
    StatesQuery.Next;
  end;
  YearsTrackBar.Value := 0;
  PopulationIndex := 0;
  PopYearLabel.Text := PopulationYears[PopulationIndex];
  SetStatePopulationDepth();
  StatesQuery.Close;
  BrazilDataBase.Close;
end;

procedure TForm2.sbDownMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
	Btn := 3;
	Timer1.Enabled := true;
end;

procedure TForm2.sbLeftMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
	Btn := 4;
	Timer1.Enabled := true;
end;

procedure TForm2.SetStatePopulationDepth;
var
  i : integer;
begin
  // use PopulationIndex to set the depth
  // for each of the states based on population
  PopQuery.ParamByName('CensusYear').Value := PopulationYears[PopulationIndex];
  PopQuery.Open;
  while not PopQuery.Eof do begin
    for i := 0 to BrazilStateMax-1 do begin
      if PopQuerySTATE_ABBREVIATION.AsString = StateArray[i].StateAbbreviation then begin
        StateArray[i].StatePath3D.Depth :=
          StrToFloat(PopQueryPOPULATION.AsString)/PopulationScaleFactor;
        StateArray[i].StatePath3D.Position.Z := -StateArray[i].StatePath3D.Depth/2
      end
    end;
    PopQuery.Next
  end;
  PopQuery.Close;
  Viewport3D1.Repaint
end;

procedure TForm2.spRightMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
	Btn := 2;
	Timer1.Enabled := true;
end;

procedure TForm2.spUpMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  Btn := 1;
	Timer1.Enabled := true;
end;

procedure TForm2.ArcDial2Change(Sender: TObject);
begin
  Layout3D1.RotationAngle.Z := ArcDial2.Value;
end;

procedure TForm2.ResetCameraButtonClick(Sender: TObject);
begin
  Camera1.Position.X := 20;
  Camera1.Position.Y := 15;
  Camera1.Position.Z := -50;
  Layout3d1.RotationAngle.X := 0;
  Layout3d1.RotationAngle.Y := 0;
  Layout3d1.Position.X := 300;
  Layout3d1.Position.Y := 100;

end;

procedure TForm2.CloseStatesButtonClick(Sender: TObject);
var
  i : integer;
begin
  for i := 0 to BrazilStateMax-1 do begin
    { How created:
      StateArray[i].Path3D := TPath3D.Create(self);
      StateArray[i].Path3D.Parent := ViewPort3D1;
      StateArray[i].Path3D.Path.Data := StatesQueryPATH_DATA.AsString;
    }
    StateArray[i].StatePath3D.Destroy;
    Viewport3D1.Repaint
  end;
  CloseStatesButton.Enabled := false;
  OpenStatesButton.Enabled := true;
  YearsTrackBar.Enabled := false;
  PopYearLabel.Enabled := false;
end;

procedure TForm2.FormCreate(Sender: TObject);
begin
  CamVector := Camera1.Position.Vector;
end;

procedure TForm2.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
	Timer1.Enabled := false;
end;

procedure TForm2.Timer1Timer(Sender: TObject);
begin
  case Btn of
    1: Layout3D1.Position.Y := Layout3D1.Position.Y - 5;
    2: Layout3D1.Position.X := Layout3D1.Position.X + 5;
    3: Layout3D1.Position.Y := Layout3D1.Position.Y + 5;
    4: Layout3D1.Position.X := Layout3D1.Position.X - 5;
  end;
end;

procedure TForm2.YearsTrackBarChange(Sender: TObject);
begin
  // Move through the years for each Brazil State's population
  PopulationIndex := trunc(YearsTrackBar.Value);
  PopYearLabel.Text := PopulationYears[PopulationIndex];
  // set depth on TPath3D to population value for each state
  SetStatePopulationDepth();
end;

procedure TForm2.tbZoomChange(Sender: TObject);
begin
  // FMX.Types3D.TPosition3D.Vector
  // System.Math.Vectors.TVector3D
	// Camera1.Position.Vector := CamVector.Scale(tbZoom.Value/20);
  Camera1.Scale.X := tbZoom.Value / 20;
  Camera1.Scale.Y := tbZoom.Value / 20;
  Camera1.Scale.Z := tbZoom.Value / 20;
end;

end.
