unit MainUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.UIConsts,System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects3D, FMX.Types3D,
  FMX.Layouts, FMX.Memo, IBDatabase, Data.DB, IBCustomDataSet, IBQuery,
  FMX.Layers3D, Data.Bind.Components, Data.Bind.DBScope, FMX.Materials,
  Datasnap.DBClient;

const
  BrazilStateMax = 27; // # of Brazil States + Brasilia
  PopulationDataYearsMax = 12;  // # of years where we have population data
  ScalingConstant = 2;  // used to scale the Path3D(s)
  PopulationScaleFactor = 1000000.0; // used to set the population depth


type
  TForm2 = class(TForm)
    Viewport3D1: TViewport3D;
    OpenStatesButton: TButton;
    Camera1: TCamera;
    Path3D1: TPath3D;
    CloseStatesButton: TButton;
    ScaledLayout1: TScaledLayout;
    Panel1: TPanel;
    Layout3D1: TLayout3D;
    Light1: TLight;
    BrazilStatesCDS: TClientDataSet;
    BrazilStatesCDSSTATE_ABBREVIATION: TStringField;
    BrazilStatesCDSSTATE_NAME: TStringField;
    BrazilStatesCDSPATH_DATA: TMemoField;
    BrazilStatesCDSSTATE_GRAPHIC: TBlobField;
    procedure OpenStatesButtonClick(Sender: TObject);
    procedure CloseStatesButtonClick(Sender: TObject);
  private
    { Private declarations }
    StateArray : array [0..BrazilStateMax-1] of record
      StateAbbreviation : string;
      StateName : string;
      StatePath3D : TPath3D;
      StateMaterialSource : TColorMaterialSource;
      // StateGraphic : TJpeg;
    end;
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

  StatesColors : array[0..BrazilStateMax-1] of TAlphaColor
    = (claBlue,claBrown,claDarkgreen,
       claCrimson,claCoral,claDarkorange,
       claSlateBlue,claDarkviolet,claForestgreen,
       claFuchsia,claGreen,claBlue,
       claIndigo,claMagenta,claMaroon,
       claNavy,claOrange,claLime,
       claPurple,claRed,claSalmon,
       claSpringgreen,claSteelBlue,claYellow,
       claTomato,claSienna,claTomato
      );

implementation

{$R *.fmx}

procedure TForm2.OpenStatesButtonClick(Sender: TObject);
var
  i : integer;
  BR : TRectF;
begin
  OpenStatesButton.Enabled := false;
  CloseStatesButton.Enabled := true;
  i := 0;
  Randomize;
  BrazilStatesCDS.LoadFromFile('BrazilStates.cds');
  BrazilStatesCDS.Active := true;
  while not BrazilStatesCDS.Eof do begin
    StateArray[i].StateAbbreviation :=BrazilStatesCDSSTATE_ABBREVIATION.AsString;
    StateArray[i].StateName := BrazilStatesCDSSTATE_NAME.AsString;
    StateArray[i].StatePath3D := TPath3D.Create(self);
    StateArray[i].StatePath3D.Projection := TProjection.pjScreen;

    StateArray[i].StatePath3D.Parent := Layout3D1;
    // TODO - have state bitmaps in the db ->StateArray[i].StateGraphic := StatesQuerySTATE_GRAPHIC.AsBytes;
    StateArray[i].StatePath3D.Path.Data := BrazilStatesCDSPATH_DATA.AsString;

    // get bounded rectangle for the state
    BR := StateArray[i].StatePath3D.Path.GetBounds;
    StateArray[i].StatePath3D.Position.X := (BR.Left+BR.Width/2)/ScalingConstant;
    StateArray[i].StatePath3D.Position.Y := (BR.Top+BR.Height/2)/ScalingConstant;
    StateArray[i].StatePath3D.Height := BR.Height/ScalingConstant;
    StateArray[i].StatePath3D.Width := BR.Width/ScalingConstant;
    StateArray[i].StatePath3D.Depth := 0;

    // Set material, back and shaft colors for the state
    StateArray[i].StateMaterialSource := TColorMaterialSource.Create(self);
    StateArray[i].StateMaterialSource.Parent := StateArray[i].StatePath3D;
    StateArray[i].StateMaterialSource.Color := StatesColors[i];
    StateArray[i].StatePath3D.MaterialSource := StateArray[i].StateMaterialSource;
    StateArray[i].StatePath3D.MaterialShaftSource := StateArray[i].StateMaterialSource;
    StateArray[i].StatePath3D.MaterialBackSource := StateArray[i].StateMaterialSource;

    Inc(i);
    BrazilStatesCDS.Next;
  end;
end;


procedure TForm2.CloseStatesButtonClick(Sender: TObject);
var
  i : integer;
begin
  BrazilStatesCDS.Active := false;
  for i := 0 to BrazilStateMax-1 do begin
    { How created:
      StateArray[i].Path3D := TPath3D.Create(self);
      StateArray[i].Path3D.Parent := ViewPort3D1;
      StateArray[i].Path3D.Path.Data := StatesQueryPATH_DATA.AsString;
    }
    StateArray[i].StatePath3D.Destroy;
    Viewport3D1.Repaint
  end;
  CloseStatesButton.Enabled := false;
  OpenStatesButton.Enabled := true;
end;

end.
