unit MobileFormUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms3D, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.StdCtrls, FMX.ListBox, FMX.Layouts, FMX.MaterialSources, FMX.Controls3D,
  FMX.Objects, FMX.Layers3D, FMX.Objects3D;

type
  TMainForm = class(TForm3D)
    Camera1: TCamera;
    DummyX: TDummy;
    DummyY: TDummy;
    DummyZ: TDummy;
    Layout3D1: TLayout3D;
    GridXY: TGrid3D;
    GridXZ: TGrid3D;
    GridYZ: TGrid3D;
    Mesh1: TMesh;
    Layer3D1: TLayer3D;
    Layout1: TLayout;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Text2: TText;
    cbGrids: TCheckBox;
    cbOpaque: TCheckBox;
    Layer3D2: TLayer3D;
    Layout2: TLayout;
    ArcDial1: TArcDial;
    Button5: TButton;
    sbDn: TSpeedButton;
    Path1: TPath;
    sbLt: TSpeedButton;
    Path3: TPath;
    sbRt: TSpeedButton;
    Path4: TPath;
    sbUp: TSpeedButton;
    Path2: TPath;
    tbZoom: TTrackBar;
    Light1: TLight;
    Mesh1MaterialSource: TLightMaterialSource;
    StyleBook1: TStyleBook;
    Text3D1MaterialSource: TLightMaterialSource;
    Label1: TLabel;
    ListBox1: TListBox;
    ListBoxItem1: TListBoxItem;
    ListBoxHeader1: TListBoxHeader;
    ListBoxItem2: TListBoxItem;
    ListBoxItem3: TListBoxItem;
    ListBoxItem4: TListBoxItem;
    ListBoxItem5: TListBoxItem;
    Text1: TText;
    ListBoxItem6: TListBoxItem;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure RotateButtonDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure RotateButtonUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure Form3DCreate(Sender: TObject);
    procedure Form3DResize(Sender: TObject);
    procedure cbGridsChange(Sender: TObject);
    procedure cbOpaqueChange(Sender: TObject);
    procedure ArcDial1Change(Sender: TObject);
    procedure tbZoomChange(Sender: TObject);
    procedure Text1Click(Sender: TObject);
    procedure Text2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure GenerateMesh(Func : Integer);
    procedure RotateButtonTimer(Sender : TObject);
    procedure ResetZoomAndRotation;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.fmx}

uses
  Math, {ParseExpr,} FMX.Platform, {Unit2,} System.UIConsts;

const
  ProVersion = False;

var
  MinF       : Double = 2000;
  MaxF       : Double = -2000;
  CamVector  : TVector3D;
  LastResult : Double = 0;
{
  MyParser   : TExpressionParser;
}
  CustFunc   : String;
  ExprIndex  : Integer;
  XVar, ZVar : Double;
  Btn   : TSpeedButton;
  Tmr   : TTimer;
  Count : Integer;




function f(Func : Integer; x, z : Double) : Double;
var
  temp : Double;
begin
  temp := x*x+z*z;
  if temp < Epsilon then
    temp := Epsilon;

  case Func of
    1 : Result := -2000*Sin(temp/180*Pi)/temp;
    2 : Result := -5*Exp(Sin(x/10)-Cos(z/10));
    3 : Result := -5*Exp(Sin(Sqrt(Abs(x)))+Cos(Sqrt(Abs(z))));
{
    4 : begin
          XVar := x;
          ZVar := z;
          try
            Result := MyParser.AsFloat[ExprIndex];
            LastResult := Result;
          except
            Result := LastResult; // Avoids singular events
          end;
        end;
}
  end;

  if Result < MinF then
    MinF := Result;
  if Result > MaxF then
    MaxF := Result;
end;


function MulDiv(nNumber, nNumerator, nDenominator: Single): Single; overload; inline;
begin
  Result := (nNumber * nNumerator) / nDenominator;
end;


function MulDiv(nNumber, nNumerator, nDenominator: Double): Double; overload; inline;
begin
  Result := (nNumber * nNumerator) / nDenominator;
end;


function GetAveCharSize(Canvas: TCanvas): TPointF;
var
  I: Integer;
  Buffer: String;
begin
  SetLength(Buffer, 52);
  for I := 1 to 26 do
    Buffer[I] := Chr((I - 1) + Ord('A'));
  for I := 27 to 52 do
    Buffer[I] := Chr((I - 27) + Ord('a'));
  Result := PointF(Canvas.TextWidth(Buffer), Canvas.TextHeight(Buffer));
  Result.X := Result.X / 52;
end;


procedure TMainForm.GenerateMesh(Func : Integer);
const
  MaxX = 30;
  MaxZ = 30;
var
  u, v : Double;
  P : array [0..3] of TPoint3D;
  d : Double;
  NP, NI : Integer;
  BMP : TBitmap;
  BMPData : TBitmapData;
  k : Integer;
begin
  Mesh1.Data.Clear;

  d := 0.5;

  NP := 0;
  NI := 0;

  // We have to set these up front. The buffers are cleared every time Length is set.

  Mesh1.Data.VertexBuffer.Length := Round(2*MaxX*2*MaxZ/d/d)*4;
  Mesh1.Data.IndexBuffer.Length := Round(2*MaxX*2*MaxZ/d/d)*6;

  BMP := TBitmap.Create(1,360);
  BMP.Map(TMapAccess.maReadWrite,BMPData);
  for k := 0 to 359 do
    BMPData.SetPixel(0,k,HSLtoRGB(k/360,0.75,0.5));
  BMP.Unmap(BMPData);

  u := -MaxX;
  while u < MaxX do begin
    v := -MaxZ;
    while v < MaxZ do begin
      // Set up the points in the XZ plane
      P[0].x := u;
      P[0].z := v;
      P[1].x := u+d;
      P[1].z := v;
      P[2].x := u+d;
      P[2].z := v+d;
      P[3].x := u;
      P[3].z := v+d;

      // Calculate the corresponding function values for Y = f(X,Z)
      P[0].y := f(Func,P[0].x,P[0].z);
      P[1].y := f(Func,P[1].x,P[1].z);
      P[2].y := f(Func,P[2].x,P[2].z);
      P[3].y := f(Func,P[3].x,P[3].z);

      with Mesh1.Data do begin
        // Set the points
        with VertexBuffer do begin
          Vertices[NP+0] := P[0];
          Vertices[NP+1] := P[1];
          Vertices[NP+2] := P[2];
          Vertices[NP+3] := P[3];
        end;

        // Map the colors
        with VertexBuffer do begin
          TexCoord0[NP+0] := PointF(0,(P[0].y+35)/45);
          TexCoord0[NP+1] := PointF(0,(P[1].y+35)/45);
          TexCoord0[NP+2] := PointF(0,(P[2].y+35)/45);
          TexCoord0[NP+3] := PointF(0,(P[3].y+35)/45);
        end;

        // Map the triangles
        IndexBuffer[NI+0] := NP+1;
        IndexBuffer[NI+1] := NP+2;
        IndexBuffer[NI+2] := NP+3;
        IndexBuffer[NI+3] := NP+3;
        IndexBuffer[NI+4] := NP+0;
        IndexBuffer[NI+5] := NP+1;
      end;

      NP := NP+4;
      NI := NI+6;

      v := v+d;
    end;
    u := u+d;
  end;

  Mesh1.Data.CalcFaceNormals;
  Mesh1MaterialSource.Texture := BMP;
  BMP.Free;
end;

procedure TMainForm.ResetZoomAndRotation;
begin
  tbZoom.Value := 20;
  DummyX.ResetRotationAngle;
  DummyY.ResetRotationAngle;
  DummyZ.ResetRotationAngle;
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  ResetZoomAndRotation;
//  Text3D1.Text := Button1.Text;
  CustFunc := '-2000*Sin((x*x+z*z)/180*Pi)/(x*x+z*z)';
  GenerateMesh(1);
  ResetZoomAndRotation;
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  ResetZoomAndRotation;
//  Text3D1.Text := Button2.Text;
  CustFunc := '-5*Exp(Sin(x/10)-Cos(z/10))';
  GenerateMesh(2);
  ResetZoomAndRotation;
end;

procedure TMainForm.Button3Click(Sender: TObject);
begin
  ResetZoomAndRotation;
//  Text3D1.Text := Button3.Text;
  CustFunc := '-5*Exp(Sin(Sqrt(Abs(x)))+Cos(Sqrt(Abs(z))))';
  GenerateMesh(3);
end;

procedure TMainForm.Button4Click(Sender: TObject);
var
  Cancelled : Boolean;
begin
  Cancelled := False;

{
  if not ProVersion then
    Cancelled := not InputQuery('Enter a custom function of X and Z','Function f(x,z) =',CustFunc);

  if ProVersion then begin
    Form2 := TForm2.Create(nil);
    Form2.Edit1.Text := CustFunc;
    Form2.Width := ClientWidth;
    Form2.Height := ClientHeight;
    Form2.Show;
    repeat
      Application.ProcessMessages;
    until not Form2.Visible;
    CustFunc := Form2.Edit1.Text;
    Cancelled := Form2.Cancelled;
    Form2.Free;
  end;

  if Cancelled then
    Exit;

  ResetZoomAndRotation;
  MyParser := TExpressionParser.Create;
  MyParser.DefineVariable('x',@XVar);
  MyParser.DefineVariable('z',@ZVar);
  MyParser.Optimize := True;
  try
    ExprIndex := MyParser.AddExpression(CustFunc);
//    Text3D1.Text := CustFunc;
    GenerateMesh(4);
  except
    on E: Exception do
      ShowMessage(E.Message);
  end;
  MyParser.Free;
}
end;

procedure TMainForm.Button5Click(Sender: TObject);
begin
  ResetZoomAndRotation;
end;


procedure TMainForm.RotateButtonDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  Count := 0;
  Btn := Sender as TSpeedButton;

  if not Assigned(Tmr) then begin
    Tmr := TTimer.Create(Self);
    Tmr.Interval := 5;
    Tmr.OnTimer := RotateButtonTimer;
  end;

  Tmr.Enabled := True;
end;

procedure TMainForm.RotateButtonTimer(Sender: TObject);
begin
  Inc(Count);
  if Btn = sbUp then
    DummyX.RotationAngle.X := DummyX.RotationAngle.X - 1;
  if Btn = sbDn then
    DummyX.RotationAngle.X := DummyX.RotationAngle.X + 1;
  if Btn = sbRt then
    DummyY.RotationAngle.Y := DummyY.RotationAngle.Y - 1;
  if Btn = sbLt then
    DummyY.RotationAngle.Y := DummyY.RotationAngle.Y + 1;
end;

procedure TMainForm.RotateButtonUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  if Assigned(Tmr) then
    Tmr.Enabled := False;
end;

procedure TMainForm.tbZoomChange(Sender: TObject);
begin
  Camera1.Position.X := CamVector.X*(20/tbZoom.Value); // 0.5 - 2.5
  Camera1.Position.Y := CamVector.Y*(20/tbZoom.Value); // 0.5 - 2.5
  Camera1.Position.Z := CamVector.Z*(20/tbZoom.Value); // 0.5 - 2.5
end;

procedure TMainForm.Text1Click(Sender: TObject);
begin
  cbGrids.IsChecked := not cbGrids.IsChecked;
end;

procedure TMainForm.Text2Click(Sender: TObject);
begin
  cbOpaque.IsChecked := not cbOpaque.IsChecked;
end;

procedure TMainForm.ArcDial1Change(Sender: TObject);
begin
  DummyZ.RotationAngle.Z := ArcDial1.Value;
end;

procedure TMainForm.cbGridsChange(Sender: TObject);
begin
  GridXY.Visible := cbGrids.IsChecked;
  GridXZ.Visible := cbGrids.IsChecked;
  GridYZ.Visible := cbGrids.IsChecked;
end;

procedure TMainForm.cbOpaqueChange(Sender: TObject);
begin
  if cbOpaque.IsChecked then
    Mesh1.Opacity := 1
  else
    Mesh1.Opacity := 0.5;
end;

{
procedure TMainForm.Edit1KeyDown(Sender: TObject; var Key: Word; var KeyChar: Char;
  Shift: TShiftState);
begin
  if Key = 13 then begin
//    Platform.HideVirtualKeyboard;
    Button4.SetFocus;
    Button4.OnClick(Self);
  end;
end;
}

procedure TMainForm.Form3DCreate(Sender: TObject);
begin
  CamVector := Camera1.Position.Vector;
  Form3DResize(Self);
end;

procedure TMainForm.Form3DResize(Sender: TObject);
begin
  Layer3D1.Position.Y := Layer3D1.Height/2;
  Layer3D1.Position.X := Layer3D1.Width/2;

  Layer3D2.Position.Y := Layer3D2.Height/2;
  Layer3D2.Position.X := ClientWidth-Layer3D2.Width/2;
end;

end.
